#' Generation of string infix operators
#' 
#' @param A The first argument
#' @param B The second argument
#' 
#' @details
#' This package generates infix operators: `&`, `&&`, `|`, `||`, \cr
#' which are almost equivalent to, resp, `\%&\%`, `\%&&\%`, `\%|\%`, `\%||\%`
#' \cr in the package lazy.tools. \cr\cr
#' The difference is that `&` has precedence over `|``  but `\%&\%`` does NOT.
#' \cr
#' Therefore, it is better to use parentheses.
#' \cr
#' \cr
#' See the descriptions of And, Or, and Not in the package lazy.tools.
#' \cr\cr
#' \code{ "a" & "b" } is equivalent to  \code{And( "a", "b" )}
#' and generates \code{"(a) & (b)"}.\cr
#' \code{"a" && "b"} is equivalent to  \code{And( "a", "b", oper="&&" )}
#' and generates \code{"(a) && (b)"}.\cr
#' \code{ "a" | "b" } is equivalent to \code{Or( "a", "b" )}
#' and generates \code{"(a) | (b)"}.\cr
#' \code{ "a" || "b"} is equivalent to  \code{Or( "a", "b", oper="||" )}
#' and generates \code{"(a) || (b)"}.\cr
#' \cr
#' Note that `&` has precedence over `|``  but `\%&\%`` does NOT.
#' \cr\cr
#' Use \code{eval( parse(text=condition) )} to evaluate the expression.
#' \cr
#' \cr
#' To remove these operators: \cr
#'  \code{ detach("package:lazy.andor", unload=TRUE) }
#'
#' @return
#' a string
#'
#' @examples
#' 
#' "a" & "b"
#' And( "a", "b" )
#' "a" | "b"
#' Or( "a", "b" )
#' 
#' "a" | "b" & "c"
#' "a" %|% "b" %&% "c"
#' 
#' "a" & "b" | "c"
#' "a" %&% "b" %|% "c"
#' 
#' @export
#'

andor <- function( A, B ){
 # dummy function
 #  # Shin-ichi Mayekawa
 # 20171007
 
 cat("\nThis is a dummy function.\n")
}

######################################################################
################## Modified Infix Operators ##########################
######################################################################
#' @rdname andor
#' @export
#'

`&` <- function( A, B=NULL ){
 # & operator
 # Shin-ichi Mayekawa
 # 20171006,07
 
 if( .Primitive("|")(is.character(A), is.character(B)) ){
  if( is.null(B) ) return( And(TRUE,A))
  else return( And(A,B) )
 }
 else{
  if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
  else return(  .Primitive("&")(A,B) )
 }
} # end of &


#' @rdname andor
#' @export
#'

`|` <- function( A, B=NULL ){
 # | operator
 # Shin-ichi Mayekawa
 # 20171006,07
 
 if( .Primitive("|")(is.character(A), is.character(B)) ){
  if( is.null(B) ) return( Or(TRUE,A))
  else return( Or(A,B) )
 }
 else{
  if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
  else return(  .Primitive("|")(A,B) )
 }
} # end of |


#' @rdname andor
#' @export
#'

`&&` <- function( A, B=NULL ){
 # & operator
 # Shin-ichi Mayekawa
 # 20171006,07
 
 if( .Primitive("|")(is.character(A), is.character(B)) ){
  if( is.null(B) ) return( And(TRUE,A, oper=" && "))
  else return( And(A,B, oper=" && ") )
 }
 else{
  if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
  else return(  .Primitive("&")(A,B) )
 }
} # end of &&


#' @rdname andor
#' @export
#'

`||` <- function( A, B=NULL ){
 # | operator
 # Shin-ichi Mayekawa
 # 20171006,07
 
 if( .Primitive("|")(is.character(A), is.character(B)) ){
  if( is.null(B) ) return( Or(TRUE,A, oper=" || "))
  else return( Or(A,B, oper=" || ") )
 }
 else{
  if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
  else return(  .Primitive("|")(A,B) )
 }
} # end of ||


